/*
* File:        sysinit.c
* Purpose:     Kinetis L Family Configuration
*              Initializes processor to a default state
*
* Notes:
*
*
*/

#include "common.h"
#include "sysinit.h"
#include "uart.h"
#include "fll.h"
/********************************************************************/




/* System clock frequency variables - Represents the current system clock
* settings
*/
int mcg_clk_hz;
int mcg_clk_khz;
int core_clk_khz;
int lrc_clk_hz;
int lrc_clk_khz;
int bus_clk_khz;
int flash_clk_khz;

int uart0_clk_khz;
int uart1_clk_khz;
int uart2_clk_khz;
int uart3_clk_khz;
int lpuart0_clk_khz;





/********************************************************************/
void sysinit (void)
{
    /* Enable all of the port clocks. These have to be enabled to configure
    * pin muxing options, so most code will need all of these on anyway.
    */
    SIM_SCGC5 |= (SIM_SCGC5_PORTA_MASK
                  |SIM_SCGC5_PORTB_MASK
                      |SIM_SCGC5_PORTC_MASK 
                          |SIM_SCGC5_PORTD_MASK
                              |SIM_SCGC5_PORTE_MASK
                                  |SIM_SCGC5_PORTF_MASK
                                      |SIM_SCGC5_PORTG_MASK
                                          |SIM_SCGC5_PORTH_MASK
                                              |SIM_SCGC5_PORTI_MASK );
    SIM_SCGC6 |= (SIM_SCGC6_PORTJ_MASK
                  |SIM_SCGC6_PORTK_MASK
                      |SIM_SCGC6_PORTL_MASK 
                          |SIM_SCGC6_PORTM_MASK);
    /* Release hold with ACKISO:  Only has an effect if recovering from VLLS1, VLLS2, or VLLS3
    if ACKISO is set you must clear ackiso before calling pll_init 
    or pll init hangs waiting for OSC to initialize
    if osc enabled in low power modes - enable it first before ack
    if I/O needs to be maintained without glitches enable outputs and modules first before ack.
    */
    if (PMC_REGSC &  PMC_REGSC_ACKISO_MASK)
        PMC_REGSC |= PMC_REGSC_ACKISO_MASK;
    
#ifdef ENABLE_CLKOUT
    // Initialize trace clk functionality
    clk_out_init();
#endif
    
    /* switch to normal run mode */
    SMC_PMCTRL &= ~SMC_PMCTRL_RUNM_MASK;
    /* wait until run mode*/
    while ( !(PMC_REGSC&PMC_REGSC_REGONS_MASK) ); 
    /* wait until run mode*/
    while ( !(SMC_PMSTAT&0x1) );
    
    MCG_SC &= ~MCG_SC_FCRDIV_MASK;
    MCG_SC |= MCG_SC_FCRDIV(FCRDIV);
    
    /*Configure Core/sys clock , Bus/flash clock*/
    SIM_CLKDIV1 = 0;
    
    SIM_CLKDIV1 = (0|SIM_CLKDIV1_CLKDIVSYS(CLKSYSDIV)|SIM_CLKDIV1_CLKDIVBUS(CLKDIVBUS));    //Core/sys clk = 8Mhz, Bus/flash clk = 4Mhz
    if(FLASHCLKMODE)
        SIM_CLKDIV1 |= SIM_CLKDIV1_FLASHCLKMODE_MASK;
    
#ifdef _FEE_24MHZ
    /*MCGOUTCLK 24Mhz*/
    FLL_Init  (FLL_MODULE_FEE_24MHZ_CONFIG);
#endif    
    /*
    * Use the value obtained from the pll_init function to define variables
    * for the core clock in kHz and also the peripheral clock. These
    * variables can be used by other functions that need awareness of the
    * system frequency.
    */
#ifdef _IRC_4MHZ
    mcg_clk_hz = CORECLK_FREQ_HZ>>((MCG_SC>>1)&0x7);
#else    
    mcg_clk_hz = CORECLK_FREQ_HZ; 
#endif   
    mcg_clk_khz = mcg_clk_hz/1000;
    
    /*Core/Platform/System clock*/
    core_clk_khz = mcg_clk_khz / (((SIM_CLKDIV1 & SIM_CLKDIV1_CLKDIVSYS_MASK) >> 28)+ 1);   
    /*Bus clock*/
    bus_clk_khz = core_clk_khz / (((SIM_CLKDIV1 & SIM_CLKDIV1_CLKDIVBUS_MASK) >> 24)+ 1);
    /*Flash clock*/
    flash_clk_khz = bus_clk_khz / (((SIM_CLKDIV1 & SIM_CLKDIV1_FLASHCLKMODE_MASK) >> 16)+ 1);
    
    uart0_clk_khz = bus_clk_khz;
    uart2_clk_khz = bus_clk_khz;
    uart1_clk_khz = core_clk_khz;
    uart3_clk_khz = core_clk_khz;
    
    
#if (TERM_PORT_NUM == 0)        //UART_0
    /* Enable the UART_TXD function on PTK_2 */
    PORTK_PCR2 = PORT_PCR_MUX(0x2); 
    /* Enable the UART_RXD function on PTK_3 */
    PORTK_PCR3 = PORT_PCR_MUX(0x2);         
    uart_init (UART0_BASE_PTR, uart0_clk_khz, TERMINAL_BAUD);
#elif (TERM_PORT_NUM == 1)      //UART_1
    /* Enable the UART_TXD function on PTI_1 */
    PORTI_PCR1 = PORT_PCR_MUX(0x2); 
    /* Enable the UART_RXD function on PTI_0 */
    PORTI_PCR0 = PORT_PCR_MUX(0x2);         
    uart_init (UART1_BASE_PTR, uart1_clk_khz, TERMINAL_BAUD);
#elif (TERM_PORT_NUM == 2)      //UART_2
    /* Enable the UART_TXD function on PTI_7 */
    PORTI_PCR7 = PORT_PCR_MUX(0x2); 
    /* Enable the UART_RXD function on PTI_6 */
    PORTI_PCR6 = PORT_PCR_MUX(0x2);         
    uart_init (UART2_BASE_PTR, uart2_clk_khz, TERMINAL_BAUD);
#elif (TERM_PORT_NUM == 3)        //UART_3        
    /* Enable the UART_TXD function on PTC_2 */
    PORTC_PCR2 = PORT_PCR_MUX(0x2); 
    /* Enable the UART_RXD function on PTC_3 */
    PORTC_PCR3 = PORT_PCR_MUX(0x2);         
    uart_init (UART3_BASE_PTR, uart3_clk_khz, TERMINAL_BAUD);
#elif (TERM_PORT_NUM == 4)      //LPUART_0
    /*LPUART clock source is from MCGIRCLK.*/
    /*Fast Internal reference clock selected. IRC4MHz*/
    MCG_C2 |= MCG_C2_IRCS_MASK;
    /*Active MCGIRCLK*/
    MCG_C1 |= MCG_C1_IRCLKEN_MASK;
    /* LPUART CLK is form MCGIRCLK*/
    SIM_CTRL_REG &= ~(SIM_CTRL_REG_LPUARTSRC_MASK);
    SIM_CTRL_REG |= SIM_CTRL_REG_LPUARTSRC(3);    
    lpuart0_clk_khz = MCGIRCLK_4MHZ/1000; //1.7Mhz 
    
    /* Enable the LPUART_TXD function on PTJ_5 */
    PORTJ_PCR5 = PORT_PCR_MUX(0x2); 
    /* Enable the LPUART_RXD function on PTJ_6 */
    PORTJ_PCR6 = PORT_PCR_MUX(0x2);

    lpuart_init (LPUART0_BASE_PTR, lpuart0_clk_khz, TERMINAL_BAUD);
#else
    #error "ERR!!! No any UART is selected !!"
#endif

}
/********************************************************************/
void enable_abort_button(void)
{
    /* Configure the PTA4 pin for its GPIO function */
    PORTA_PCR4 = PORT_PCR_MUX(0x1) | PORT_PCR_PE_MASK | PORT_PCR_PS_MASK; // GPIO is alt1 function for this pin
    
    /* Configure the PTA4 pin for rising edge interrupts */
    PORTA_PCR4 |= PORT_PCR_IRQC(0x9); 
    
    /* Enable the associated IRQ in the NVIC */
#ifndef CMSIS
    enable_irq(INT_PTx-16);
#else
    // NVIC_EnableIRQ(PORTA_IRQn);
#endif
}
/********************************************************************/

/********************************************************************/
void clk_out_init(void)
{
    
    /* Enable the CLKOUT function on PTF7 (alt3 function) */
    PORTF_PCR7 = PORT_PCR_MUX(0x3);
    
    /*
    000 Disable
    001 Gated core clk
    010 Bus clk
    011 LPU clk from PMC
    100 IRC clk from MCG
    101 Muxed 32Khz source
    110 Mhz Oscillator external reference clk
    111 PLL clk output from MCG
    */
    SIM_CTRL_REG |= SIM_CTRL_REG_CLKOUT(1);        
    
    
}
