/*
 * File:        firebird.h
 * Purpose:     Kinetis tower CPU card definitions
 *
 * Notes:
 */

#ifndef __FIREBIRD_H__
#define __FIREBIRD_H__


/********************************************************************/

/* Global defines to use for all Tower cards */
#define DEBUG_PRINT

/* Uncomment this next line if you desire the clock output (note: in conflict with UART1)*/
#define ENABLE_CLKOUT

//#define _IRC_4MHZ
#define _FEE_24MHZ

//#define LOW_POWER_TEST  //To test VLPR, LLS, VLLx

/*
* Input Clock Info
*/
#if (defined _FEE_24MHZ) && (defined _IRC_4MHZ)
  #error "ERR !! Can't define 4Mhz and 24Mhz simultaneously ."
#endif

/*
* Configure Core/Sys clock 2Mhz, Bus clock 1Mhz, Flash clock 1Mhz in Low Power Mode.
*/
#ifdef  LOW_POWER_TEST  
  
  #ifdef _FEE_24MHZ
    #undef _FEE_24MHZ
  #endif
  #ifndef _IRC_4MHZ
    #define _IRC_4MHZ
  #endif
      
  #define FCRDIV          (0)     //Low-frequency Internal Reference Clock Divider
  #define CLKSYSDIV       (1)     //System Clock Divider
  #define CLKDIVBUS       (1)     //Bus Clock Divider, 00 sys:bus= 1:1, 01 sys:bus=2:1, 10 sys:bus=3:1, 11 sys:bus=4:1
  #define FLASHCLKMODE    (0)     //Flash Clock Mode, 0 flash clock is the same as BUS clk; 1 is a half of BUS clk.
#else
  #define FCRDIV          (0)     //Low-frequency Internal Reference Clock Divider
  #define CLKSYSDIV       (0)     //System Clock Divider
  #define CLKDIVBUS       (0)     //Bus Clock Divider, 00 sys:bus= 1:1, 01 sys:bus=2:1, 10 sys:bus=3:1, 11 sys:bus=4:1
  #define FLASHCLKMODE    (0)     //Flash Clock Mode, 0 flash clock is the same as BUS clk; 1 is a half of BUS clk.
#endif
      
#if (defined _FEE_24MHZ)
  #define CORECLK_FREQ_HZ         (24000000)      //24Mhz
#elif (defined _IRC_4MHZ)
  #define CORECLK_FREQ_HZ         (4000000)      //4Mhz ,
#else
  #error "ERR !! Undefine clock source. "
#endif      

#define MCGIRCLK_4MHZ           (4000000)    //MCGIRCLK for LPUART0, untrimmed chip
/* Serial Port Info */
/**************************************************************************
   * Note:
   * 
   * Because of the changes to the UART modules, we can no longer define
   * the TERM_PORT as a base pointer.  The uart functions have been modified 
   * accommodate this change.  Now, TERM_PORT_NUM must be defined as the 
   * number of the UART port desired to use
   *
   * TERM_PORT_NUM = 0  -- This allows you to use UART0;   
   * TERM_PORT_NUM = 1  -- This allows you to use UART1;  
   * TERM_PORT_NUM = 2  -- This allows you to use UART2;
   * TERM_PORT_NUM = 3  -- This allows you to use UART3;
   * TERM_PORT_NUM = 4  -- This allows you to use LPUART0;
   *
   *************************************************************************/

/* Uses UART0 for both Open SDA */

#define TERM_PORT_NUM       2
     
#define TERMINAL_BAUD       9600//115200
       
#undef  HW_FLOW_CONTROL

#if (TERM_PORT_NUM > 4)
#error "ERR !!! UART port error !!"
#endif

#endif /* __TOWER_H__ */
