#include "common.h"

#define PAGE_SIZE 256

#define CMD_GET_ID      0x9f
#define CMD_SECTOR_ERASE 0x20
#define CMD_DEV_ERASE   0x60
#define CMD_PROGRAM     0x02
#define CMD_READ        0x0B  //FAST READ , 0X03 READ
#define CMD_GET_SR      0x05
#define CMD_SET_SR      0x01
#define CMD_WRITE_EN    0x06
#define CMD_WRITE_DIS   0x04
#define WEL             0X00
#define WIP             0X01
void serial_flash_init(void)
{
    SIM_SCGC4 |= SIM_SCGC4_SPI0_MASK|SIM_SCGC4_SPI1_MASK; 		   //Enable SPI0 and SPI1 Clock gate 
    SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK|SIM_SCGC5_PORTB_MASK|SIM_SCGC5_PORTC_MASK|SIM_SCGC5_PORTD_MASK|SIM_SCGC5_PORTE_MASK|SIM_SCGC5_PORTF_MASK|SIM_SCGC5_PORTG_MASK|SIM_SCGC5_PORTH_MASK|SIM_SCGC5_PORTI_MASK; 	 
  
    PORTD_PCR1 &= ~PORT_PCR_MUX_MASK;        //SPI0
    PORTD_PCR1 |= PORT_PCR_MUX(1) |0X03;			  //Use PTD1 as SPI0_CS  // configure it as the GPIO 
    //PORTD_PCR1 |= PORT_PCR_MUX(3) |0X03;			  //Use PTD1 as SPI0_CS    
    PORTD_PCR2 &= ~PORT_PCR_MUX_MASK;
    PORTD_PCR2 |= PORT_PCR_MUX(3) |0X03;			  //Use PTD2 as SPI0_SCK 
    PORTD_PCR3 &= ~PORT_PCR_MUX_MASK;
    PORTD_PCR3 |= PORT_PCR_MUX(3) |0X03;			  //Use PTD3 as SPI0_MOSI  
    PORTD_PCR4 &= ~PORT_PCR_MUX_MASK;
    PORTD_PCR4 = PORT_PCR_MUX(3) |0X03;			          //Use PTD4 as SPI0_MISO
    

    GPIOD_PDDR |=  0X02;
    GPIOD_PDOR |=  0X02; // output high
    
    
   //=======================Configure SPI0 
    		SPI0_C1 |= SPI_C1_MSTR_MASK;	  //-----master----bus clock is 12.5Mhz--0.08us--
		
		SPI0_BR = 0x43;  //SPPR = 4, SPR = 3, bps div = (SPPR+1)*2^(SPR+1) = 80,----300Khz
		//SPI0_BR = 0x40; //bps div = 10,---------0.8us
		//SPI0_BR = 0x30; //bps div = 8,----------0.64us
		//SPI0_BR = 0x10; //bps div = 4,----------0.32us
		//SPI0_BR = 0x00; //bps div = 2,----------0.16us----6.125Mhz
		//SPI0_BR = 0x54; //bps div = 192,----------15.36us
		//SPI0_BR = 0x77; //bps div = 2048,----------163.84us 
		//SPI0_BR = 0x78;	  //bps div = 8*2^9 = 4096---327.68us
		SPI0_C1 |= SPI_C1_SSOE_MASK;      //|SPI0_C1_CPOL_MASK|SPI0_C1_LSBFE_MASK;
		
		//SPI0_C1 |= SPI_C1_CPHA_MASK;
		SPI0_C1 &= (~SPI_C1_CPHA_MASK);

		SPI0_C1 &= (~SPI_C1_CPOL_MASK);
		//SPI0_C1 |= SPI_C1_CPOL_MASK;

		SPI0_C1 &= (~SPI_C1_LSBFE_MASK);
		//SPI0_C1 |= SPI_C1_LSBFE_MASK;

		SPI0_C1 &= (~SPI_C1_SPIE_MASK);   //Disable RX interrrupt
  		//SPI0_C1 |= SPI_C1_SPIE_MASK;	   //enable RX interrrupt 		  
		SPI0_C1 &= (~SPI_C1_SPTIE_MASK);	   //Disable the transmit interrupt 
 		//SPI0_C1 |= SPI_C1_SPTIE_MASK;	   //Enable the transime interrupt

		SPI0_C2 |= SPI_C2_MODFEN_MASK;
                
                SPI0_C1 |= SPI_C1_SPE_MASK;
}


void hal_spi_transfe_start(void)
{
    GPIOD_PCOR |=  0X02; // output LOW CS LOW
}

void hal_spi_transfe_stop(void)
{

   GPIOD_PDOR |=  0X02; // output high
}
uint8 hal_spi_transfer_one_byte(uint8 v)
{
   int dummy =0;
   char buff=0;

   while ((SPI0_S & SPI_S_SPTEF_MASK) == 0)
   {
	dummy++;
    }
   dummy = SPI0_S;
   SPI0_DL = v; 
   
   while ((SPI0_S & SPI_S_SPRF_MASK) == 0);
   buff = SPI0_DL;
   return buff;         
}

static void write_enable(void)
{
    hal_spi_transfe_start();
    hal_spi_transfer_one_byte(CMD_WRITE_EN);
    hal_spi_transfe_stop();
}

static uint8 get_sr(void)
{
    uint8 v;
    hal_spi_transfe_start();
    hal_spi_transfer_one_byte(CMD_GET_SR);
    v = hal_spi_transfer_one_byte(0x00);
    hal_spi_transfe_stop();
    return v;
}

static void spi_wait(uint8 CMD)
{
  if(CMD == WEL)
    while(get_sr()&0x02 != 0x02); // wait until WEL bit =1
  else if(CMD == WIP)
    while(get_sr()&0x01 != 0x00); // wait until WIP bit =0
}

void print_buf(void *buf, uint16 size)
{
    int i;
    uint8 *p = (uint8*)buf;
    for(i=0;i<size;i++)
    {
        printf("0x%.2x,", p[i]);
        if(((i+1)&0x3) == 0)
            printf("  ");
        if(((i+1)&0x7) == 0)
            printf("\n");
    }

    printf("\n");
}


void hal_spi_dev_flash_erase_sector(uint8 addr)
{
    write_enable();
     spi_wait(WEL);
    hal_spi_transfe_start();
    hal_spi_transfer_one_byte(CMD_SECTOR_ERASE);
    hal_spi_transfer_one_byte(addr>>16);
    hal_spi_transfer_one_byte(addr>>8);
    hal_spi_transfer_one_byte(addr>>0);
    hal_spi_transfe_stop();
      spi_wait(WIP);   
}
void hal_spi_dev_flash_program_page(uint8 addr, char *buf)
{
    int i;
    write_enable();
     spi_wait(WEL);
    hal_spi_transfe_start();
    hal_spi_transfer_one_byte(CMD_PROGRAM);
    hal_spi_transfer_one_byte(addr>>16);
    hal_spi_transfer_one_byte(addr>>8);
    hal_spi_transfer_one_byte(addr>>0);
    for(i=0;i<(PAGE_SIZE-1);i++)
        hal_spi_transfer_one_byte(buf[i]);
    hal_spi_transfer_one_byte(buf[i]);
    hal_spi_transfe_stop();
    spi_wait(WIP);
    
      
}

void hal_spi_dev_flash_read_page(uint8 addr, char *buf)
{
    int i;
    hal_spi_transfe_start();
    hal_spi_transfer_one_byte(CMD_READ);
    hal_spi_transfer_one_byte(addr>>16);
    hal_spi_transfer_one_byte(addr>>8);
    hal_spi_transfer_one_byte(addr>>0);
    hal_spi_transfer_one_byte(0x00); // dummy byte
    for(i=0;i<(PAGE_SIZE-1);i++)
        buf[i] = hal_spi_transfer_one_byte(0x00);
    buf[i] = hal_spi_transfer_one_byte(0x00);
    hal_spi_transfe_stop();
}
void external_spi_test(void)
{
  static char buf[256];
      int i;
     serial_flash_init();
     hal_spi_dev_flash_erase_sector(0);
     printf("reading page...\n");
     hal_spi_dev_flash_read_page(0,buf);
     print_buf(buf,PAGE_SIZE);

    printf("programing a page...\n");
    for(i=0;i<256;i++)
        buf[i] = i;//+0x30
    hal_spi_dev_flash_program_page(0,buf);

    printf("clearing buffer..\n");
    for(i=0;i<256;i++)
        buf[i] = 0;

    printf("reading page...\n");
    hal_spi_dev_flash_read_page(0,buf);
    print_buf(buf,PAGE_SIZE);
    
    printf("demo end.\n");
}

